<?php

namespace FiloBlu\Storelocator\Block\Adminhtml\Stores\Edit\Tab;

class Store extends \Magento\Backend\Block\Widget\Form\Generic implements \Magento\Backend\Block\Widget\Tab\TabInterface {

    /**
     * @var \Magento\Store\Model\System\Store
     */
    protected $_systemStore;
    protected $_countryFactory;
    protected $_googleApi;
    // protected $config = null;

    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Data\FormFactory $formFactory
     * @param \Magento\Cms\Model\Wysiwyg\Config $wysiwygConfig
     * @param \Magento\Store\Model\System\Store $systemStore
     * @param array $data
     */
    public function __construct(
    \Magento\Backend\Block\Template\Context $context,
    \Magento\Framework\Registry $registry,
    \Magento\Framework\Data\FormFactory $formFactory,
    \Magento\Store\Model\System\Store $systemStore,
    \Magento\Directory\Model\Config\Source\Country $countryFactory,
    \FiloBlu\Storelocator\Model\Google\Api $googleApi,
    // \Magento\Framework\App\Config\ScopeConfigInterface $config,
            array $data = []
    ) {
        $this->_systemStore = $systemStore;
        $this->_countryFactory = $countryFactory;
        $this->_googleApi = $googleApi;
        $this->config = $context->getScopeConfig();

        parent::__construct($context, $registry, $formFactory, $data);
    }

    /**
     * Init form
     *
     * @return void
     */
    protected function _construct() {
        parent::_construct();
        $this->setId('storelocator_store_form');
        $this->setTitle(__('Store Information'));
    }

    /**
     * Prepare form
     *
     * @return $this
     */
    protected function _prepareForm() {
        /** @var \FiloBlu\Storelocator\Model\Stores $model */
        $model = $this->_coreRegistry->registry('storelocator_store');

        $form = $this->_formFactory->create();

        $form->setHtmlIdPrefix('storelocator_store_');

        // ## General Information ##
        $fieldset = $form->addFieldset(
                'base_fieldset', ['legend' => __('General Information'), 'class' => 'fieldset-wide']
        );

        $fieldset->addType('image', '\FiloBlu\Storelocator\Block\Adminhtml\Stores\Helper\Image');

        if ($model->getStoreId()) {
            $fieldset->addField('store_id', 'hidden', ['name' => 'store_id']);
        }

        $fieldset->addField(
                'title', 'text', [
            'name' => 'title',
            'label' => __('Title'),
            'title' => __('Title'),
            'required' => true
                ]
        );

        $fieldset->addField(
            'store_filename_prefix', 'text', [
                'name' => 'store_filename_prefix',
                'label' => __('Store image file prefix'),
                'title' => __('Store image file prefix')
            ]
        );

        $fieldset->addField(
                'enabled', 'select', [
            'label' => __('Status'),
            'title' => __('Status'),
            'name' => 'enabled',
            'required' => true,
            'options' => ['1' => __('Enabled'), '0' => __('Disabled')]
                ]
        );

        $fieldset->addField(
                'is_enabled_for_pickup_in_store', 'select', [
            'label' => __('Enabled For Pickup In Store'),
            'title' => __('Enabled For Pickup In Store'),
            'name' => 'is_enabled_for_pickup_in_store',
            'required' => true,
            'options' => ['1' => __('Enabled'), '0' => __('Disabled')]
                ]
        );

        $fieldset->addField(
            'is_enabled_for_return_in_store', 'select', [
                'label' => __('Enabled For Return In Store'),
                'title' => __('Enabled For Return In Store'),
                'name' => 'is_enabled_for_return_in_store',
                'required' => true,
                'options' => ['1' => __('Enabled'), '0' => __('Disabled')]
            ]
        );

        $fieldset->addField(
            'is_enabled_shop_details', 'select', [
                'label' => __('Enabled Shop details'),
                'title' => __('Enabled Shop details'),
                'name' => 'is_enabled_shop_details',
                'required' => true,
                'options' => ['1' => __('Enabled'), '0' => __('Disabled')]
            ]
        );


        if (!$model->getId()) {
            $model->setData('enabled', '1');
        }

        if (!$this->_storeManager->isSingleStoreMode()) {
            $field = $fieldset->addField(
                    'store_ids', 'multiselect', [
                'name' => 'store_ids[]',
                'label' => __('Assign to Store Views'),
                'title' => __('Assign to Store Views'),
                'required' => true,
                'values' => $this->_systemStore->getStoreValuesForForm(false, true)
                    ]
            );
            $renderer = $this->getLayout()->createBlock(
                    'Magento\Backend\Block\Store\Switcher\Form\Renderer\Fieldset\Element'
            );
            $field->setRenderer($renderer);
        }

        $fieldset->addField(
                'image', 'image', array(
            'name' => 'image',
            'label' => __('Image'),
            'title' => __('Image')
                )
        );

        $fieldset->addField(
                'marker_image', 'image', array(
            'name' => 'marker_image',
            'label' => __('Marker Image'),
            'title' => __('Marker Image')
                )
        );

        // ## Address ##
        $fieldset = $form->addFieldset(
                'address_fieldset', ['legend' => __('Address'), 'class' => 'fieldset-wide']
        );

        $fieldset->addField(
                'street', 'text', [
            'name' => 'street',
            'label' => __('Street'),
            'title' => __('Street')
                ]
        );

        $fieldset->addField(
                'postcode', 'text', [
            'name' => 'postcode',
            'label' => __('Postcode'),
            'title' => __('Postcode')
                ]
        );

        $fieldset->addField(
                'city', 'text', [
            'name' => 'city',
            'label' => __('City'),
            'title' => __('City')
                ]
        );

        // $model->setCountryId($model->getCountry());

        $optionsc = $this->_countryFactory->toOptionArray();
        $fieldset->addField(
                'country', 'select', [
            'name' => 'country',
            'label' => __('Country'),
            'title' => __('Country'),
            // 'onchange' => 'getstate(this)',
            'values' => $optionsc,
                ]
        );

        $regionField = $fieldset->addField(
            'region',
            'text',
            ['name' => 'region', 'label' => __('State/Province'), 'required' => true]
        );

        $regionUpdaterBlock = $this->getLayout()->createBlock('FiloBlu\Storelocator\Block\Adminhtml\Renderer\Region');
        $regionUpdaterBlock->setRegionIdValue($model->getRegionId());
        $regionUpdaterBlock->setRegionValue($model->getRegion());
        $regionField->setRenderer($regionUpdaterBlock);

        $fieldset->addField(
                'additional_text', 'FiloBlu\Storelocator\Block\Adminhtml\Stores\Helper\Editor', [
            'name' => 'additional_text',
            'label' => __('Additional Text'),
            'title' => __('Additional Text'),
                ]
        );
        $fieldset->addField(
            'additional_text_2', 'FiloBlu\Storelocator\Block\Adminhtml\Stores\Helper\Editor', [
                'name' => 'additional_text_2',
                'label' => __('Additional Text 2'),
                'title' => __('Additional Text 2'),
            ]
        );


        $fieldset->addField(
            'store_external_iframe_appointment', 'FiloBlu\Storelocator\Block\Adminhtml\Stores\Helper\Editor', [
                'name' => 'store_external_iframe_appointment',
                'label' => __('External Iframe Appointment'),
                'title' => __('External Iframe Appointment'),
            ]
        );


        // ## Contacts ##
        $fieldset = $form->addFieldset(
                'contacts_fieldset', ['legend' => __('Contacts'), 'class' => 'fieldset-wide']
        );

        $fieldset->addField(
                'phone_1', 'text', [
            'name' => 'phone_1',
            'label' => __('Phone 1'),
            'title' => __('Phone 1')
                ]
        );

        $fieldset->addField(
                'phone_2', 'text', [
            'name' => 'phone_2',
            'label' => __('Phone 2'),
            'title' => __('Phone 2')
                ]
        );

        $fieldset->addField(
                'fax', 'text', [
            'name' => 'fax',
            'label' => __('Fax'),
            'title' => __('Fax')
                ]
        );

        $fieldset->addField(
                'email', 'text', [
            'name' => 'email',
            'label' => __('Email'),
            'title' => __('Email')
                ]
        );

        $fieldset->addField(
                'website_url', 'text', [
            'name' => 'website_url',
            'label' => __('Website Url'),
            'title' => __('Website Url')
                ]
        );


        // ## Geolocation ##
        $fieldset = $form->addFieldset(
                'geolocation_fieldset', ['legend' => __('Geolocation data'), 'class' => 'fieldset-wide']
        );

        $fieldset->addField(
                'latitude', 'text', [
            'name' => 'latitude',
            'label' => __('Latitude'),
            'title' => __('Latitude')
                ]
        );

        $fieldset->addField(
                'longitude', 'text', [
            'name' => 'longitude',
            'label' => __('Longitude'),
            'title' => __('Longitude')
                ]
        );

        $fieldGMap = $fieldset->addField(
                'google_map', 'label', [
            'latitude' => $model->getLatitude(),
            'longitude' => $model->getLongitude(),
            'required' => false,
                ]
        );
        $renderer = $this->getLayout()->createBlock(
                'FiloBlu\Storelocator\Block\Adminhtml\Stores\Form\Renderer\Fieldset\GoogleMap'
        );
        $fieldGMap->setRenderer($renderer);


        // ## Google My Business ##
        $gmb_enabled = $this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_enabled');

        if ($gmb_enabled) {
            $fieldset = $form->addFieldset(
                    'gmb_fieldset', ['legend' => __('Google My Business'), 'class' => 'fieldset-wide']
            );

            $language = "it-IT";
            $country = "IT";

            if ($model->getId()) {
                if ($model->getGmbLanguageCode()) {
                    $language = $model->getGmbLanguageCode();
                }
                if ($model->getCountry()) {
                    $country = $model->getCountry();
                }
            }


            $fieldset->addField(
                    'gmb_cat', 'select', [
                'name' => 'gmb_cat',
                'label' => __('Category'),
                'title' => __('Category'),
                'required' => true,
                'options' => $this->_googleApi->getCategoryList($country, $language)
                    ]
            );

            $fieldset->addField(
                    'gmb_language_code', 'text', [
                'name' => 'gmb_language_code',
                'label' => __('Language'),
                'title' => __('Language'),
                'required' => true,
                    ]
            );
        }


        $form->setValues($model->getData());
//        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Prepare label for tab
     *
     * @return string
     */
    public function getTabLabel() {
        return __('Store');
    }

    /**
     * Prepare title for tab
     *
     * @return string
     */
    public function getTabTitle() {
        return $this->getTabLabel();
    }

    /**
     * Can show tab in tabs
     *
     * @return boolean
     */
    public function canShowTab() {
        return true;
    }

    /**
     * Tab is hidden
     *
     * @return boolean
     */
    public function isHidden() {
        return false;
    }

}
