<?php

namespace FiloBlu\Storelocator\Model\Plugin\Rma\Source;

use FiloBlu\Storelocator\Helper\Data;

class Status
{
    CONST STATE_RECEIVED_IN_STORE = 'return_received_in_store';
    CONST STATE_RECEIVED_IN_STORE_PHRASE = 'Return Received in Store';
    /**
     * @var \Magento\Rma\Model\Item\Attribute\Source\StatusFactory 
     */
    protected $_statusFactory;
    /**
     * @var Data
     */
    protected $helper;

    public function __construct(
        \Magento\Rma\Model\Item\Attribute\Source\StatusFactory $statusFactory,
        Data $helper
    )
    {
        $this->_statusFactory = $statusFactory;
        $this->helper = $helper;
    }

    public function beforeGetItemLabel(\Magento\Rma\Model\Rma\Source\Status $subject, $state) {

        if (!$this->helper->getReturnToStoreEnabled()) {
            return [$state];
        }

        if ($state == self::STATE_RECEIVED_IN_STORE) {
            return [__(self::STATE_RECEIVED_IN_STORE_PHRASE)];
        }

        return [$state];
    }

    public function afterGetAllOptionsForGrid(\Magento\Rma\Model\Rma\Source\Status $subject, $result) {

        if (!$this->helper->getReturnToStoreEnabled()) {
            return $result;
        }

        if (!isset($result[self::STATE_RECEIVED_IN_STORE])) {
            $result[self::STATE_RECEIVED_IN_STORE] = __(self::STATE_RECEIVED_IN_STORE_PHRASE);
        }

        return $result;

    }

    public function aroundGetStatusByItems(\Magento\Rma\Model\Rma\Source\Status $subject, callable $proceed, $itemStatusArray) {

        if (!$this->helper->getReturnToStoreEnabled()) {
            return $proceed($itemStatusArray);
        }

        if (!is_array($itemStatusArray) || empty($itemStatusArray)) {
            throw new \Magento\Framework\Exception\LocalizedException(__('This is the wrong RMA item status.'));
        }

        $itemStatusArray = array_unique($itemStatusArray);

        /** @var $itemStatusModel \Magento\Rma\Model\Item\Attribute\Source\Status */
        $itemStatusModel = $this->_statusFactory->create();

        foreach ($itemStatusArray as $status) {
            if (!$itemStatusModel->checkStatus($status)) {
                throw new \Magento\Framework\Exception\LocalizedException(__('This is the wrong RMA item status.'));
            }
        }

        $itemStatusToBits = [
            \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PENDING => 1,
            \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_AUTHORIZED => 2,
            \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_DENIED => 4,
            \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_RECEIVED => 8,
            \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_APPROVED => 16,
            \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_REJECTED => 32,
            \FiloBlu\Storelocator\Model\Plugin\Rma\Source\Status::STATE_RECEIVED_IN_STORE => 64
        ];
        $rmaBitMaskStatus = 0;
        foreach ($itemStatusArray as $status) {
            $rmaBitMaskStatus += $itemStatusToBits[$status];
        }

        if ($rmaBitMaskStatus == 1) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PENDING;
        } elseif ($rmaBitMaskStatus == 2) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_AUTHORIZED;
        } elseif ($rmaBitMaskStatus == 4) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_CLOSED;
        } elseif ($rmaBitMaskStatus == 5) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PENDING;
        } elseif ($rmaBitMaskStatus > 2 && $rmaBitMaskStatus < 8) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PARTIAL_AUTHORIZED;
        } elseif ($rmaBitMaskStatus == 8) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_RECEIVED;
        } elseif ($rmaBitMaskStatus >= 9 && $rmaBitMaskStatus <= 15) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_RECEIVED_ON_ITEM;
        } elseif ($rmaBitMaskStatus == 16) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PROCESSED_CLOSED;
        } elseif ($rmaBitMaskStatus == 20) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PROCESSED_CLOSED;
        } elseif ($rmaBitMaskStatus >= 17 && $rmaBitMaskStatus <= 31) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_APPROVED_ON_ITEM;
        } elseif ($rmaBitMaskStatus == 32) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_CLOSED;
        } elseif ($rmaBitMaskStatus == 36) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_CLOSED;
        } elseif ($rmaBitMaskStatus >= 33 && $rmaBitMaskStatus <= 47) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_REJECTED_ON_ITEM;
        } elseif ($rmaBitMaskStatus == 48) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PROCESSED_CLOSED;
        } elseif ($rmaBitMaskStatus == 52) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_PROCESSED_CLOSED;
        } elseif ($rmaBitMaskStatus > 48 && $rmaBitMaskStatus < 64) {
            return \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_APPROVED_ON_ITEM;
        } elseif ($rmaBitMaskStatus == 64) {
            return \FiloBlu\Storelocator\Model\Plugin\Rma\Source\Status::STATE_RECEIVED_IN_STORE;
        }
    }

    public function afterGetButtonDisabledStatus(\Magento\Rma\Model\Rma\Source\Status $subject, $result, $status = null) {
        if (!$this->helper->getReturnToStoreEnabled()) {
            return $result;
        }

        if ($status !== self::STATE_RECEIVED_IN_STORE) {
            return $result;
        }

        return false;
    }
}